Attribute VB_Name = "mDrivers"
Option Explicit
    
' Declare needed API calls
    Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (ByVal Destination As Long, ByVal source As Long, ByVal length As Long)

' Driver type for enumeration of D3D driver...
    Public Type tD3DDriver
        DESC    As String                         ' Driver description
        NAME    As String                         ' Driver name
        GUID    As Byte                           ' Unique interface ID for accessing driver
        GUID1   As Byte                           ' ...
        GUID2   As Byte                           ' ...
        GUID3   As Byte                           ' ...
        GUID4   As Byte                           ' ...
        GUID5   As Byte                           ' ...
        GUID6   As Byte                           ' ...
        GUID7   As Byte                           ' ...
        GUID8   As Byte                           ' ...
        GUID9   As Byte                           ' ...
        GUID10  As Byte                           ' ...
        GUID11  As Byte                           ' ...
        GUID12  As Byte                           ' ...
        GUID13  As Byte                           ' ...
        GUID14  As Byte                           ' ...
        GUID15  As Byte                           ' ...
        DEVDESC As D3DDEVICEDESC                  ' Device description for use by D3DRM
        HDW     As Boolean                        ' Device is hardware
        EMU     As Boolean                        ' Device is software-emulated
        RGB     As Boolean                        ' Device has rgb caps
        MONO    As Boolean                        ' Device has mono ramp caps
    End Type
    
' Driver Array for driver detection ...
    Public G_dD3DDriver() As tD3DDriver                 ' Drivers for use with Direct3DRM
    Public G_nD3DDriverCount As Integer                 ' Count of drivers
    Global G_dD3DSelectedDriver As tD3DDriver           ' Driver to use
    
' ENUMDEVICECALLBACK: Enumerates Device drivers for Direct3D
Public Function EnumDeviceCallback(nGUID As Long, nDDDesc As Long, nDDName As Long, dHALDDDesc As D3DDEVICEDESC, dHELDDDesc As D3DDEVICEDESC, nOptions As Long) As Long

    ' Setup local variables ...
    
        Dim L_nTemp(256) As Byte                      ' Temporary array for name and guid translation
        Dim L_nChar As Byte                           ' Temporary charactar for name translation
        Dim L_nIndex As Long                          ' Variable to run through temp array
        Dim L_dD3DDriver As tD3DDriver
        
    ' Process current driver ...
    
        ' Inspect current driver
        With L_dD3DDriver
            
        ' Set driver capabilities...
          
            ' Decide if hardware supports color model and enable HAL or HEL support properly
            If dHALDDDesc.dcmColorModel Then
              .DEVDESC = dHALDDDesc
              .HDW = True
            Else
              .DEVDESC = dHELDDDesc
            End If

            ' Set RGB capability
            .RGB = (.DEVDESC.dcmColorModel = D3DCOLOR_RGB)
            ' Set emulation mode
            .EMU = (Not .HDW)
            ' Set mono ramp mode
            .MONO = (Not .RGB)
            
        ' Decide if driver fits application needs ...
        
            ' Exit without naming/enabling driver if no RGB support
'            If Not .RGB Then
'                EnumDeviceCallback = DDENUMRET_OK
'                Exit Function
'            End If
        
            ' Exit without naming/enabling driver if no support for minimum 8 bit color depth
'            If ((.DEVDESC.dwDeviceRenderBitDepth And DDBD_8) = 0) And ((.DEVDESC.dwDeviceRenderBitDepth And DDBD_16) = 0) And ((.DEVDESC.dwDeviceRenderBitDepth And DDBD_24) = 0) And ((.DEVDESC.dwDeviceRenderBitDepth And DDBD_32) = 0) Then
'                EnumDeviceCallback = DDENUMRET_OK
'                Exit Function
'            End If
            
        End With
            
    ' DRIVER ACCEPTED ...
    
        ' Prepare space for driver
        G_nD3DDriverCount = G_nD3DDriverCount + 1
        ReDim Preserve G_dD3DDriver(G_nD3DDriverCount)
        
        ' Add driver to driver list
        G_dD3DDriver(G_nD3DDriverCount) = L_dD3DDriver
        
        ' Set driver info
        With G_dD3DDriver(G_nD3DDriverCount)
    
            ' Copy GUID data into temporary array
            CopyMemory VarPtr(L_nTemp(0)), VarPtr(nGUID), 16
            
            ' Set GUID data into driver structure
            .GUID = L_nTemp(0)
            .GUID1 = L_nTemp(1)
            .GUID2 = L_nTemp(2)
            .GUID3 = L_nTemp(3)
            .GUID4 = L_nTemp(4)
            .GUID5 = L_nTemp(5)
            .GUID6 = L_nTemp(6)
            .GUID7 = L_nTemp(7)
            .GUID8 = L_nTemp(8)
            .GUID9 = L_nTemp(9)
            .GUID10 = L_nTemp(10)
            .GUID11 = L_nTemp(11)
            .GUID12 = L_nTemp(12)
            .GUID13 = L_nTemp(13)
            .GUID14 = L_nTemp(14)
            .GUID15 = L_nTemp(15)
            
            ' Copy driver name into temporary array
            CopyMemory VarPtr(L_nTemp(0)), VarPtr(nDDName), 255
              
            ' Parse name of driver
            For L_nIndex = 0 To 255
                L_nChar = L_nTemp(L_nIndex)
                If L_nChar < 32 Then Exit For
                .NAME = .NAME + Chr(L_nChar)
            Next
            
            ' Copy driver Description into temporary array
            CopyMemory VarPtr(L_nTemp(0)), VarPtr(nDDDesc), 255
              
            ' Parse description of driver
            For L_nIndex = 0 To 255
                L_nChar = L_nTemp(L_nIndex)
                If L_nChar < 32 Then Exit For
                .DESC = .DESC + Chr(L_nChar)
            Next
            
        End With
            
    ' Return success
    EnumDeviceCallback = DDENUMRET_OK

End Function

